(************************************************************************
*    This File is Part of the AHM Triton Tools                          *
*    for Borland Delphi and C++ Builder.                                *
*************************************************************************
* Copyright  1995-2000 Alexander H. Mehlhorn (AHM) and Tritontools.com *
* All Rights reserved.                                                  *
*                                                                       *
*    For more Information contact us at                                 *
*    E-Mail : support@tritontools.com                                   *
*    URL : http://www.tritontools.com                                   *
*                                                                       *
*    AHM is copyright by Alexander H. Mehlhorn                          *
*    All Parts of this Collection were developed,                       *
*    designed and implemented by AHM                                    *
*                                                                       *
*    Under no Circumstances may you distribute this file                *
*    in this or any other format. This File is supplied                 *
*    as Evaluation of Source only. No Delphi Components may be developed*
*    using extracts or subsets of this code without expressed permission*
*    by AHM.                                                            *
*                                                                       *
*************************************************************************)
unit AHMTMRUMenu;

interface

uses Classes,SysUtils,Registry,Windows,Menus,AHMTStoreData,ahmclasses;

type
  TAHMMRUItem = class(TMenuItem)
  private
  public
    ItemNumber: Byte;
    constructor Create(AOwner: TComponent); override;
  end;
  TAHMMRUItemClick = procedure(Sender: TObject;Filename: string) of Object;
  TAHMMRUMenu = class(TAHMComponent)
  private
    Fnumbering : Boolean;
    FMaxItems: Byte;
    FRemoveOnClick: boolean;
    FAsSubMenu: boolean;
    FInsertSeparator : Boolean;
    FSubMenuName: string;
    FFileMenu: TMenuItem;
    FMRUStore :  TAHMDataStore;
    FMenuItems: TStringList;
    FOnClick: TAHMMRUItemClick;
  protected
    procedure Loaded; override;
    procedure SetMaxItems(Val: Byte);
    procedure SetFileMenu(Val: TMenuItem);
    procedure SetAsSubMenu(Val: boolean);
    procedure SetInsertSeparator(Val: boolean);
    procedure SetSubMenuName(Val: string);
    procedure MRUClicked(Sender: TObject);
    procedure PopulateMenu; virtual;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(Owner: TComponent); override;
    destructor Destroy; override;
    function Load: boolean;
    function Save: boolean;
    procedure RemoveAllItems;
    procedure Clear;
    procedure Delete(Itemname: string);
    procedure Insert(Itemname : String);
    property Items: TStringList read FMenuItems;
  published
    property MaxItems: Byte read FMaxItems write SetMaxItems default 10;
    property RemoveOnClick: boolean read FRemoveOnClick write FRemoveOnClick default True;
    property AsSubMenu: boolean read FAsSubMenu write SetAsSubMenu default False;
    property Separator : boolean read FInsertSeparator write SetInsertSeparator default True;
    property SubMenuname: string read FSubMenuName write SetSubMenuName;
    property OnItemClick: TAHMMRUItemClick read FOnClick write FOnClick;
    property AttachToMenu: TMenuItem read FFileMenu write SetFileMenu;
    Property MRUStore : TAHMDataStore read fMRUstore write fMRUstore;
    Property Numbering : Boolean read FNumbering write FNumbering;
  end;

implementation

constructor TAHMMRUItem.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ItemNumber := 0;
end;

constructor TAHMMRUMenu.Create(Owner: TComponent);
begin
  inherited Create(Owner);
  FMaxItems := 10;
  FRemoveOnClick := True;
  FAsSubMenu := True;
  FInsertSeparator:=True;
  SubMenuName := 'Reopen';
  FMenuItems := TStringList.Create;
  FMenuItems.Sorted := false;
  FMRUStore:=TAHMDataStore.Create;
  FMRUStore.IniFile.Section := 'MRU Items';
  FMRUStore.Registry.Section := 'MRU Items';
end;

destructor TAHMMRUMenu.Destroy;
begin
  if not(csDesigning in ComponentState) then Save;
  FOnClick:=nil;
  FMenuItems.Free;
  FMRUStore.Free;
  inherited Destroy;
end;

procedure TAHMMRUMenu.Loaded;
begin
  if not (csDesigning in ComponentState) then
  Load;
end;

procedure TAHMMRUMenu.SetMaxItems(Val: Byte);
begin
  if FMaxItems = Val then exit;
  if Val < FMaxItems then
  begin
    while FMenuItems.Count > Val do FMenuItems.Delete(FMenuItems.Count-1);
    PopulateMenu;
  end;
  FMaxItems := Val;
end;

procedure TAHMMRUMenu.SetFileMenu(Val: TMenuItem);
begin
  RemoveAllItems;
  FFileMenu := Val;
  PopulateMenu;
end;

procedure TAHMMRUMenu.SetAsSubMenu(Val: boolean);
begin
  if FAsSubMenu = Val then exit;
  FAsSubMenu := Val;
  PopulateMenu;
end;

procedure TAHMMRUMenu.SetInsertSeparator(Val: boolean);
begin
 If Val=FInsertSeparator then exit;
 FInsertSeparator:=Val;
 PopulateMenu;
end;

procedure TAHMMRUMenu.SetSubMenuName(Val: string);
begin
  if FSubMenuName = Val then exit;
  FSubMenuName := Val;
  if FAsSubMenu then PopulateMenu;
end;

procedure TAHMMRUMenu.MRUClicked(Sender: TObject);
var ClickItem: string;
begin
  with Sender as TAHMMRUItem do
  begin
    if assigned(FOnClick) then ClickItem := FMenuItems[ItemNumber-1];
    if FRemoveOnClick then
    begin
      FMenuItems.Delete(ItemNumber-1);
      PopulateMenu;
    end;
    if assigned(FOnClick) then FOnClick(Sender, ClickItem);
  end;
end;

procedure TAHMMRUMenu.Insert(Itemname: string);
var
  i: integer;
begin
  i := FMenuItems.IndexOf(Itemname);
  if i > -1 then FMenuItems.Move(i,0) else
  begin
    FMenuItems.Add(Itemname);
    while FMenuItems.Count > FMaxItems do
    FMenuItems.Delete(0);
  end;
  PopulateMenu;
end;

procedure TAHMMRUMenu.PopulateMenu;
var
  Offset,
  x: integer;
  NewItem: TAHMMRUItem;
  ParentMenu,AddMenu: TMenuItem;
begin
  if (FFileMenu = NIL) then exit;
  RemoveAllItems;
  if (FMenuItems.Count = 0) then exit;
  if FFileMenu.Count <> 0 then
  begin
    Offset := FFileMenu.Count;
    ParentMenu := FFileMenu;
  end else begin
    Offset := FFileMenu.MenuIndex;
    ParentMenu := FFileMenu.Parent;
  end;
  If FInsertSeparator then
  begin
    NewItem := TAHMMRUItem.Create(ParentMenu);
    NewItem.Caption := '-';
    ParentMenu.Insert(Offset, NewItem);
    inc(Offset);
  end;
  if FAsSubMenu then begin
    AddMenu := TAHMMRUItem.Create(ParentMenu);
    AddMenu.Caption := FSubMenuName;
    ParentMenu.Insert(Offset, AddMenu);
    Offset := 0;
  end else AddMenu := ParentMenu;
  for x := 0 to FMenuItems.Count - 1 do begin
    NewItem := TAHMMRUItem.Create(AddMenu);
  if FNumbering then NewItem.Caption :='&'+IntToStr(x+1)+' '+FMenuItems[x]
                else NewItem.Caption :=FMenuItems[x];
    NewItem.ItemNumber := x + 1;
    NewItem.OnClick := MRUClicked;
    AddMenu.Insert(Offset, NewItem);
    inc(Offset);
  end;
end;

procedure TAHMMRUMenu.RemoveAllItems;
var
  x: integer;
  DeleteItem,
  ParentMenu: TMenuItem;
begin
  if FFileMenu = NIL then exit;
  if FFileMenu.Count <> 0 then ParentMenu := FFileMenu
                          else ParentMenu := FFileMenu.Parent;
  if ParentMenu = NIL then exit;
  for x := ParentMenu.Count-1 downto 0 do
  begin
    if (ParentMenu[x] is TAHMMRUItem) then
    begin
      DeleteItem := ParentMenu[x];
      ParentMenu.Delete(x);
      DeleteItem.Free;
    end;
  end;
end;

procedure TAHMMRUMenu.Delete(Itemname: string);
begin
  if FMenuItems.IndexOf(Itemname) > -1 then
  begin
    FMenuItems.Delete(FMenuItems.IndexOf(Itemname));
    PopulateMenu;
  end;
end;

procedure TAHMMRUMenu.Clear;
begin
  RemoveallItems;
  FMenuItems.Clear;
  Save;
end;

function TAHMMRUMenu.Load: boolean;
var I: integer;
begin
  result:=false;
  if not(FMRUStore.Enabled) then exit;
  MRUStore.ReadStrs(FMenuItems);
  for I := 0 to FMenuItems.Count-1 do
  FMenuItems[I] := copy(FMenuItems[I],Pos('=',FMenuItems[I])+1,
                        Length(FMenuItems[I])-Pos('=',FMenuItems[I])+2);
  PopulateMenu;
  Result := TRUE;
end;

function TAHMMRUMenu.Save: boolean;
var
  x: integer;
begin
  result := false;
  if not(FMRUStore.Enabled) then exit;
  FMRUStore.Erase;
  for x :=0 to FMenuItems.Count-1 do MRUStore.WriteStr('MRU'+IntToStr(x), Items[x]);
  result := true;
end;

procedure TAHMMRUMenu.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FFileMenu) and
     (csDesigning in ComponentState) then
  FFileMenu := NIL;
end;

end.

