(************************************************************************
*    This File is Part of the AHM Triton Tools                          *
*    for Borland Delphi and C++ Builder.                                *
*************************************************************************
* Copyright  1995-2000 Alexander H. Mehlhorn (AHM) and Tritontools.com *
* All Rights reserved.                                                  *
*                                                                       *
*    For more Information contact us at                                 *
*    E-Mail : support@tritontools.com                                   *
*    URL : http://www.tritontools.com                                   *
*                                                                       *
*    AHM is copyright by Alexander H. Mehlhorn                          *
*    All Parts of this Collection were developed,                       *
*    designed and implemented by AHM                                    *
*                                                                       *
*    Under no Circumstances may you distribute this file                *
*    in this or any other format. This File is supplied                 *
*    as Evaluation of Source only. No Delphi Components may be developed*
*    using extracts or subsets of this code without expressed permission*
*    by AHM.                                                            *
*                                                                       *
*************************************************************************)
unit AHMTSystemMenu;
interface

uses
  SysUtils,Windows, Messages, Classes, Graphics,
  Forms, Dialogs,ahmclasses,Menus,ahmtbackground,AHMTFormHook;
const
  Isloaded : Boolean = False;
type
  TAHMTypeCast =(onForm,onApplication);
  TAHMMenuType=(asItem,asSeparator,asBitmap);
  TAHMSystemMenu = class(TAHMHookComponent)
  private
    MyHandle : THandle;
    ftc : TAHMTypecast;
    fpos,fid : Integer;
    fcgly,fgly,fbitm : TBitmap;
    fname : String;
    fcheck,fenab,fradio : boolean;
    ftype : TAHMMenuType;
    fctrue,fcfalse,fnclick : TNotifyevent;
    procedure SetBitmap(value : TBitmap);
    procedure SetGlyph(value : TBitmap);
    procedure SetCGlyph(value : TBitmap);
    procedure Setcheck(value : Boolean);
    procedure Setenabled(value : Boolean);
  protected
    procedure resetall(value : Boolean);
  public
    procedure Hooked(var AMsg: TMessage); override;
    constructor Create(AOwner: TComponent);  override;
    procedure loaded;  override;
    destructor  Destroy; override;
  published
    property PositionOnMenu : Integer read fpos write fpos;
    property UniqueMenuID : Integer read Fid write fid;
    property MenuName : String read fname write fname;
    property RadioItem : Boolean read fradio write fradio;
    property MenuItemType : TAHMMenuType read ftype write ftype;
    property AttachTo : TAHMTypeCast read ftc write ftc;
    property Checked : Boolean read fcheck write Setcheck;
    property Enabled : Boolean read fenab write Setenabled;
    property Bitmap : TBitmap read fbitm write SetBitmap;
    property CheckedGlyph : TBitmap read fgly write Setglyph;
    property UnCheckedGlyph : TBitmap read fcgly write Setcglyph;
    property OnClickToTrue : TNotifyevent read fctrue write fctrue;
    Property OnClickToFalse : TNotifyevent read fcfalse write fcfalse;
    Property OnClickNormal : TNotifyevent read fnclick write fnclick;
  end;

implementation

constructor TAHMSystemMenu.Create(AOwner: TComponent);
begin
  fcheck:=false;
  inherited Create(AOwner);
  fbitm:=TBitmap.create;
  fgly:=TBitmap.create;
  fcgly:=TBitmap.create;
end;

procedure TAHMSystemMenu.Loaded;
var  SystemMenu : HMenu;
begin
  inherited Loaded;
  if ftc=onForm then
    MyHandle := OwnerHandle else
    MyHandle := Application.Handle;
  if not(csdesigning in Componentstate) then
  begin
      SystemMenu:=GetSystemMenu(MyHandle,False);
      if ftype=asItem then InsertMenu(SystemMenu,fpos,MF_ByPosition+MF_String,fid,pchar(fname)) else
      if ftype=asSeparator then InsertMenu(SystemMenu,fpos,MF_ByPosition+MF_Separator,fid,pchar(fname)) else
      if ftype=asBitmap then InsertMenu(SystemMenu,fpos,MF_ByPosition+MF_Bitmap,fid,pchar(fbitm.handle));
      if fcheck and (fgly<>nil) then SetMenuItemBitmaps(SystemMenu,fid,MF_ByCommand, FcGly.Handle, FGly.Handle)
      else if fcgly<>nil then SetMenuItemBitmaps(SystemMenu,fid,MF_ByCommand, FcGly.Handle, FcGly.Handle)
  end;
  Isloaded:=true;
  Setcheck(fcheck);
  setenabled(fenab);
end;

destructor TAHMSystemMenu.Destroy;
begin
  fbitm.free;
  fcgly.free;
  fgly.free;
  inherited destroy;
end;

procedure TAHMSystemMenu.SetBitmap(value : TBitmap);
begin
  fbitm.assign(value);
end;

procedure TAHMSystemMenu.SetGlyph(value : TBitmap);
begin
  fgly.assign(value);
end;

procedure TAHMSystemMenu.SetCGlyph(value : TBitmap);
begin
  fcgly.assign(value);
end;

procedure TAHMSystemMenu.SetCheck(value : Boolean);
begin
  fcheck:=value;
 if isloaded then Resetall(fcheck);
end;

procedure TAHMSystemMenu.Resetall(value : boolean);
var  SystemMenu : HMenu;
begin
  SystemMenu:=GetSystemMenu(MyHandle,False);
  if value then CheckMenuItem(SystemMenu,fid,mf_ByCommand+mf_checked)
  else CheckMenuItem(SystemMenu,fid,mf_ByCommand+mf_unchecked);
  DrawmenuBar(MyHandle);
end;

procedure TAHMSystemMenu.SetEnabled(value : Boolean);
var  SystemMenu : HMenu;
begin
  fenab:=value;
  if IsLoaded then
  begin
    SystemMenu:=GetSystemMenu(MyHandle,False);
    if value then EnableMenuItem(SystemMenu,fid,mf_ByCommand+mf_enabled)
    else EnableMenuItem(SystemMenu,fid,mf_ByCommand+mf_grayed);
    DrawmenuBar(MyHandle);
  end;
end;

procedure TAHMSystemMenu.Hooked(var AMsg: TMessage);
begin
  if not(csdesigning in Componentstate) then
  if (AMsg.Msg = WM_SYSCOMMAND) then
  begin
    if (AMsg.wParam = fid) and ((ftype=asItem) or (ftype=asBitmap)) then
    begin
      if fradio then
      begin
        if fcheck then
        begin
          if assigned(fcfalse) then fcfalse(self);
        end
        else
        begin
          if assigned(fctrue) then fctrue(self);
        end;
        Setcheck(not(fcheck));
      end
      else
      begin
        if assigned(fnclick) then fnclick(self);
      end;
    end;
  end;
  inherited Hooked(AMsg);
end;

end.

