
{ ***************************************************************************** }
{ TFindFile Component                                                           }
{ ***************************************************************************** }
{ Copyright(C) 1997 - 2001 by a.hua, All Rights Reserved !                      }
{ Please visit my site : http://www.a-hua.com                                   }
{ Email : a-hua@163.com                                                         }
{ ***************************************************************************** }
{ Problem ???? GIVE ME A EMAIL !!!!!!!!                                         }
{ ***************************************************************************** }

unit FindFile;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, DesignEditors,
  DesignIntf, DesignWindows, Forms, Dialogs, StdCtrls, FFForm;

type
  TExtStr = string;
  TDirStr = string;

  TDirStrProperty = class(TPropertyEditor)
  public
    function GetValue: string; override;
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

  TDirFiles = class
  private
    FFileList: TStringList;
    FDirList: TStringList;
    FDirectory: TDirStr;
    FFileExt: TExtStr;
  protected
    procedure CreateVariables;
    procedure DestroyVariables;
    procedure FindDirs;
    procedure FindFiles;
    procedure SetFileExt(Value: TExtStr);
  public
    constructor Create(Directory: TDirStr);
    destructor Destroy; override;
    procedure FindIt;
    property FileList: TStringList read FFileList;
    property DirList: TStringList read FDirList;
    property FileExt: TExtStr read FFileExt write SetFileExt;
  end;

  TFindFile = class(TComponent)
  private
    FFileExt: TExtStr;
    FDirectory: TDirStr;
    FFileList: TStringList;
    FListBox: TCustomListBox;
    procedure SetFileExt(Value: TExtStr);
    procedure SetDirectory(Value: TDirStr);
    procedure SetFileList(Value: TCustomListBox);
    procedure ChangeList;
    procedure CreateVariables;
    procedure DestroyVariables;
    procedure FindInDirectory(Directory: TDirStr);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Search;
    property FileList: TStringList read FFileList;
  published
    property FileExt: TExtStr read FFileExt write SetFileExt;
    property Directory: TDirStr read FDirectory write SetDirectory;
    property FileListBox: TCustomListBox read FListBox write SetFileList;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TFindFile]);
  RegisterPropertyEditor(TypeInfo(TDirStr), TFindFile, 'Directory', TDirStrProperty);
end;

procedure TFindFile.SetFileExt(Value: TExtStr);
begin
  FFileExt := Value;
  ChangeList;
end;

constructor TFindFile.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  CreateVariables;
end;

destructor TFindFile.Destroy;
begin
  DestroyVariables;
  inherited Destroy;
end;

procedure TFindFile.SetDirectory(Value: TDirStr);
begin
  FDirectory := Value;
  ChangeList;
end;

procedure TFindFile.CreateVariables;
begin
  FFileList := TStringList.Create;
end;

procedure TFindFile.DestroyVariables;
begin
  FFileList.Free;
end;

procedure TFindFile.Search;
begin
  FFileList.Clear;
  FindInDirectory(FDirectory);
end;

procedure TFindFile.FindInDirectory(Directory: TDirStr);
var
  DirFiles: TDirFiles;
  i: Integer;
begin
  DirFiles := TDirFiles.Create(Directory);
  DirFiles.FileExt := FFileExt;
  DirFiles.FindIt;
  for i := 0 to DirFiles.FileList.Count - 1 do
    FFileList.Add(Directory + '\' + DirFiles.FileList[i]);
  for i := 0 to DirFiles.DirList.Count - 1 do
    FindInDirectory(Directory + '\' + DirFiles.DirList[i]);
  DirFiles.Free;
end;

procedure TFindFile.Notification(AComponent: TComponent; Operation: TOperation);
begin
  if (AComponent = FListBox) and (Operation = opRemove) then FListBox := nil;
end;

procedure TFindFile.SetFileList(Value: TCustomListBox);
begin
  FListBox := Value;
  ChangeList;
end;

procedure TFindFile.ChangeList;
begin
  if FListBox <> nil then
  begin
    Screen.Cursor := crHourGlass;
    Search;
    FListBox.Items.Assign(FileList);
    Screen.Cursor := crDefault;
  end;
end;

constructor TDirFiles.Create(Directory: TDirStr);
begin
  inherited Create;
  CreateVariables;
  FDirectory := Directory;
end;

destructor TDirFiles.Destroy;
begin
  DestroyVariables;
  inherited Destroy;
end;

procedure TDirFiles.CreateVariables;
begin
  FFileList := TStringList.Create;
  FDirList := TStringList.Create;
end;

procedure TDirFiles.DestroyVariables;
begin
  FFileList.Free;
  FDirList.Free;
end;

procedure TDirFiles.FindIt;
begin
  FindDirs;
  FindFiles;
end;

procedure TDirFiles.FindDirs;
var
  Over: Integer;
  SearchRec: TSearchRec;
begin
  FDirList.Clear;
  Over := FindFirst(FDirectory + '\*.*', faDirectory, SearchRec);
  while Over = 0 do
  begin
    if (SearchRec.Name <> '.') and (SearchRec.Name <> '..') and
       ((SearchRec.Attr and faDirectory) = faDirectory) then
      FDirList.Add(SearchRec.Name);
    Over := FindNext(SearchRec);
  end;
  FindClose(SearchRec);
end;

procedure TDirFiles.FindFiles;
var
  Over: Integer;
  PathStr: string;
  SearchRec: TSearchRec;
begin
  FFileList.Clear;
  if FFileExt = '' then PathStr := '*.*'
                   else PathStr := '*.' + FFileExt;
  PathStr := FDirectory + '\' + PathStr;
  Over := FindFirst(PathStr, faArchive and faSysFile and faHidden and faReadOnly, SearchRec);
  while Over = 0 do
  begin
    FFileList.Add(SearchRec.Name);
    Over := FindNext(SearchRec);
  end;
  FindClose(SearchRec);
end;

procedure TDirFiles.SetFileExt(Value: TExtStr);
begin
  FFileExt := Value;
end;

function TDirStrProperty.GetValue: string;
begin
  Result := GetStrValue;
end;

function TDirStrProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paReadOnly];
end;

procedure TDirStrProperty.Edit;
var
  DirectoryDialog: TDirectoryDialog;
  DirectoryString: string;
begin
  DirectoryDialog := TDirectoryDialog.Create(Application);
  if DirectoryDialog.ShowModal = mrOk then
  begin
    DirectoryString := DirectoryDialog.DirectoryList.Directory;
    if DirectoryDialog.DirectoryList.Directory[Length(DirectoryDialog.DirectoryList.Directory)] = '\' then
      Delete(DirectoryString, Length(DirectoryString), 1);
    SetStrValue(DirectoryString);
  end;
  DirectoryDialog.Free;
end;

end.
