unit WidentOTAUtils;

interface uses ToolsApi;

  function  OTUGetCurrentSourceEditor(out Editor: IOTASourceEditor): boolean;
  function  OTUGetCurrentEditView(out EditView: IOTAEditView): boolean;
  function  OTUGetCurrentSource(var CursorPosition: integer;
    var CursorLine, CursorCharIndex: integer): string;

 { IOTAEditView.PosToCharPos is a buggy function (its "ancestor" from Old API too),
   therefore it is neccessary to work with the full source and use a self convert
   function (FindCharPos here) }

  procedure OTUSetCurrentSourcePosition(const Source: string; Position: integer);

implementation uses Windows, WIdentUtils;

function  OTUGetCurrentSourceEditor(out Editor: IOTASourceEditor): boolean;
var
  CurrentModule: IOTAModule;
  i: integer;
begin
  Editor:=nil;
  Result:=false;

  CurrentModule:=(BorlandIDEServices as IOTAModuleServices).CurrentModule;
  if CurrentModule<>nil then
    for i:=0 to CurrentModule.GetModuleFileCount-1 do
      if CurrentModule.GetModuleFileEditor(i).QueryInterface(IOTASourceEditor, Editor) = S_OK then
      begin
        Result:=true;
        break;
      end;
end;

function  OTUGetCurrentEditView(out EditView: IOTAEditView): boolean;
var
  SourceEditor: IOTASourceEditor;
begin
  Result:=false;
  if OTUGetCurrentSourceEditor(SourceEditor) then
    if SourceEditor.EditViewCount>0 then
    begin
      EditView:=SourceEditor.EditViews[0];
      Result:=true;
    end;
end;

function OTUGetCurrentSource(var CursorPosition: integer; var CursorLine, CursorCharIndex: integer): string;
const
  QuerySize = 16000;
var
  SourceEditor: IOTASourceEditor;
  EditView: IOTAEditView;
  EditReader: IOTAEditReader;
  ReadSize, Position: Longint;
  FirstTime: boolean;
  EditPos: TOTAEditPos;
  CharPos: TOTACharPos;
begin
  Result:='';
  CursorPosition:=0;
  
  if OTUGetCurrentSourceEditor(SourceEditor) then
  begin
    EditReader:=SourceEditor.CreateReader;

    FirstTime:=true;
    Position:=0;

    while (FirstTime) or (ReadSize=QuerySize) do
    begin
      SetLength(Result, Position+QuerySize);
      ReadSize:=EditReader.GetText(Position, @Result[Position+1], QuerySize);
      Inc(Position, ReadSize);
      FirstTime:=false;
    end;

    SetLength(Result, Position);

    if SourceEditor.EditViewCount>0 then
    begin
      EditView:=SourceEditor.EditViews[0];
      EditPos:=EditView.CursorPos;
      EditView.ConvertPos(true, EditPos, CharPos);
      CursorLine:=CharPos.Line;
      CursorCharIndex:=CharPos.CharIndex;
      CursorPosition:=EditView.CharPosToPos(CharPos)+1;
    end;
  end;
end;

procedure OTUSetCurrentSourcePosition(const Source: string; Position: integer);
var
  EditView: IOTAEditView;
  CharPos: TOTACharPos;
  EditPos, wasTopPos, wasEditPos: TOTAEditPos;
  newTopLine, Line, CharIndex: integer;
begin
  if OTUGetCurrentEditView(EditView) then
  begin
    wasTopPos:=EditView.TopPos;
    wasEditPos:=EditView.CursorPos;

    FindCharPos(Source, Position, Line, CharIndex);
    CharPos.Line:=Line;
    CharPos.CharIndex:=CharIndex; 
    
    EditView.ConvertPos(false, EditPos, CharPos);

    EditView.CursorPos := EditPos;

    newTopLine:=EditPos.Line-(wasEditPos.Line-wasTopPos.Line);
    if newTopLine<1 then
      newTopLine:=1;

    EditPos.Line:=newTopLine;
    EditPos.Col:=1;
    EditView.TopPos := EditPos;
    EditView.Paint;
  end;
end;

end.
