unit WidentUtils;

interface { WidentEx}

type
  TGetTextProc = function(ZeroBasedPosition: Longint;
    Buffer: PChar; Count: Longint): Longint of object;

  function  CurrentIdent(const Source: string; CurPos: integer;
    var Pos, Len: integer): boolean;
  function FindNeibIdent(const Source: string;
    CurPos: integer; Prev: boolean; var StartPos, Len: integer): boolean;
  function  FindTextIdent(Id: string; const Source: string;
    LastPos: integer; Prev: boolean; var Pos: integer): boolean;
  function  FindIdentAtPos(const Source: string; CurPos: integer; Prev: boolean;
    var Pos, Len: integer; var Ident: string): boolean;
  procedure FindCharPos(const Source: string;
    Pos: integer; var Line, CharIndex: integer);

implementation uses SysUtils;

function CurrentIdent(const Source: string; CurPos: integer;
  var Pos, Len: integer): boolean;
const
  IdentChars = ['a'..'z','A'..'Z','_', '0'..'9'];
begin
  Result:=false;

  while CurPos>=1 do
    if Source[CurPos] in IdentChars then
    begin
      dec(CurPos);
      Result:=true;
    end else
      if (not Result) and (CurPos>=2) then
        if Source[CurPos-1] in IdentChars then
        begin
          Dec(CurPos, 2);
          Result:=true;
        end else
          break
      else
        break;

  if Result then
  begin
    Pos:=CurPos+1;
    Inc(CurPos, 2);
    while (CurPos>=1) and (CurPos<=Length(Source)) do
      if Source[CurPos] in IdentChars then
        inc(CurPos)
      else
        break;

    Len:=CurPos-Pos;
  end;
end;

function FindNeibIdent(const Source: string;
  CurPos: integer; Prev: boolean; var StartPos, Len: integer): boolean;
begin
  Result:=false;
  while (CurPos>=1) and (CurPos<=Length(Source)) do
    if CurrentIdent(Source, CurPos, StartPos, Len) then
    begin
      Result:=true;
      break;
    end else
      if Prev then
        Dec(CurPos)
      else
        Inc(CurPos)
end;

function  isStrCaseIns(const Str: string; Pos: integer; const SubStr: string): boolean;
var
  i: integer;
begin
  result:=false;
  if Pos+Length(SubStr)-1<=Length(Str) then
  begin
    for i:=1 to Length(SubStr) do
      if Upcase(Str[Pos+i-1])<>UpCase(SubStr[i]) then
        exit;
  end else
    exit;
  result:=true;
end;

function FindTextIdent(Id: string; const Source: string;
  LastPos: integer; Prev: boolean; var Pos: integer): boolean;
const
  IdentChars = ['a'..'z','A'..'Z','_', '0'..'9'];
var
  StartPos: integer;

  function GoNext: boolean;
  begin
    if Prev then
      Dec(StartPos)
    else
      Inc(StartPos);
    Result:=(StartPos>=1) and (StartPos<=Length(Source));
  end;

begin
  Result:=false;

  if Id<>'' then
  begin
    Id:=UpperCase(Id);
    StartPos:=LastPos;

    while GoNext do
      if UpCase(Source[StartPos])=Id[1] then
        if IsStrCaseIns(Source, StartPos, Id) then
          if ((StartPos-1<1) or (not (Source[StartPos-1] in IdentChars))) and
             ((StartPos+Length(Id)>Length(Source)) or (not (Source[StartPos+Length(Id)] in IdentChars))) then
          begin
            Pos:=StartPos;
            Result:=true;
            break;
          end;
  end;
end;

function FindIdentAtPos(const Source: string; CurPos: integer; Prev: boolean;
  var Pos, Len: integer; var Ident: string): boolean;
var
  StartPos: integer;
  Id: string;
begin
  Result:=false;

  if CurrentIdent(Source, CurPos, StartPos, Len) then
  begin
    Id:=Copy(Source, StartPos, Len);
    Result:=FindTextIdent(Id, Source, StartPos, Prev, Pos);
    Ident:=Id;
  end;
end;

procedure FindCharPos(const Source: string;
  Pos: integer; var Line, CharIndex: integer);
var
  i: integer;
begin
  Line:=1;
  CharIndex:=0;

  i:=1;
  while i<Pos do
  begin
    if Source[i]=#13 then
    begin
      if (i+1<Pos) and (Source[i+1]=#10) then
      begin
        inc(i);
      end;
      Inc(Line);
      CharIndex:=0;
    end else
      Inc(CharIndex);
    Inc(i);
  end;
end;

end.
