unit uScriptParser;

{
 Rodolfo d'Ettorre
 rodolfo@viper.net.au
 members.tripod.com/~rodolfo_3
}

interface

uses
  Windows, Messages, SysUtils, Classes, Dialogs, ComObj;

type

 TLogEvent = procedure(sLine: String) of object;

  TAnIndex = class
  public
    FName : string;
    FUnique : boolean;
    FRequired : boolean;
    FPrimaryKey : boolean;
    FFieldList : String;
  end;

  TAField = record
    FName : string;
    FType : integer;
    FSize : integer;
    FRequired : integer;
    FAllowZeroLength : integer;
    FAttributes : integer;
    FDefaultValue : string;
  end;



  TTableProcessor = class
  private
    FsDaoVersion : string;   //'35' or '36'
    FsDatabaseFile : string;
    FsScriptFileName : string;
    FScript : TStringList;
    FsCurrentTableName : string;
    FTableList : TStringList;
    FFieldList : TStringList;
    FIndexList : TStringList;
    FIndexObjectsList : TStringList;
    FbModifyFields : boolean;
    FbModifyIndexes : boolean;
    FAccess : Variant;
    FDatabase : Variant;
    FTable : Variant;
    FIndex : Variant;
    FField : Variant;
    FOnLogMessage : TLogEvent;
    procedure ProcessTable(ATable : String);
    function TableExists(ATable : string) : boolean;
    procedure CreateATable(ATable : string);
    procedure GetFieldsAndIndexes(ATable : string);
    function GetFieldProperties(AFieldLine : string) : TAField;
    function GetAParameter(ALine : string; var AParam : string) : string;
    procedure EmptyIndexObjects;
    procedure GetIndexes;
    procedure ModifyTable(ATable : string);
    procedure AddNewIndexes(ATable : string);
    procedure AddNewFields(ATable : string);
    procedure ModifyExistingFields(ATable : string);
    procedure CreateDatabase(DatabaseName : string);
  public
    constructor create(DatabaseFile, ScriptFileName, DaoVer : string);
    destructor destroy;override;
    procedure ProcessFiles;
    property OnLogMessage : TLogEvent read FOnLogMessage write FOnLogMessage;
    property TableList : TStringList read FTableList;
    property ModifyFields : boolean read FbModifyFields write FbModifyFields;
    property ModifyIndexes : boolean read FbModifyIndexes write FbModifyIndexes;
  end;

implementation

constructor TTableProcessor.create(DatabaseFile, ScriptFileName, DaoVer : string);
begin
  inherited create;
  FsDaoVersion := DaoVer;
  FsDatabaseFile := DatabaseFile;
  FsScriptFileName := ScriptFileName;
  FScript := TStringList.create;
  FsCurrentTableName := '';
  FTableList := TStringList.create;
  FFieldList := TStringList.create;
  FIndexList := TStringList.create;
  FbModifyFields := False;
  FbModifyIndexes := False;
  FIndexObjectsList := TStringList.create;
  FOnLogMessage := nil;
end;

destructor TTableProcessor.destroy;
begin
  FScript.Free;
  FTableList.Free;
  FFieldList.Free;
  FIndexList.Free;
  EmptyIndexObjects;
  inherited
end;

procedure TTableProcessor.CreateDatabase(DatabaseName : string);
var
  avary:olevariant;
  wrkDefault :Variant;
  ABase : variant;
  CreateOptions : String;
Begin
 CreateOptions:=Format(';LANGID=%s;CP=%s;COUNTRY=%s',['0x0402','1251','0']);
  avary := 0;
  if  FsDaoVersion = '35' then
    avary := $00000020
  else
    avary := $00000040;
  wrkDefault := FAccess.Workspaces[0];
  try
    ABase := wrkDefault.CreateDatabase(DatabaseName,CreateOptions, avary);
    if assigned(FOnLogMessage) then
      FOnLogMessage('Created Database ' + DatabaseName);
    ABase.Close;  
  except
    on e:exception do
    begin
      if assigned(FOnLogMessage) then
      begin
        FOnLogMessage('Could not create Database ' + DatabaseName);
        FOnLogMessage(e.message);
      end;
      raise;
    end;
  end;
end;

procedure TTableProcessor.ProcessFiles;
var
  i : integer;
begin
  FScript.Clear;
  FScript.LoadFromFile(FsScriptFileName);
  FTableList.Clear;
  for i := 0 to FScript.Count - 1 do
  begin
    if Pos('Table:', FScript.Strings[i]) >0 then
    begin
      FTableList.Add(Trim(copy(FScript.Strings[i], 7, 100)));
    end;
  end;
  FAccess := CreateOleObject('Dao.DBEngine.' + FsDaoVersion);
  if not Fileexists(FsDatabaseFile) then
    CreateDatabase(FsDatabaseFile);
  FDatabase := FAccess.OpenDatabase(FsDatabaseFile);
  try
    for i := 0 to FTableList.Count - 1 do
      ProcessTable(FTableList.Strings[i]);
  finally
    FDatabase.Close;
  end;
end;

procedure TTableProcessor.ProcessTable(ATable : String);
begin
  GetFieldsAndIndexes(ATable);
  if not TableExists(ATable) then
  begin
    CreateATable(ATable);
  end else
  begin
    ModifyTable(ATable);
  end;
end;

function TTableProcessor.TableExists(ATable : string) : boolean;
begin
  try
    FTable := FDatabase.TableDefs[ATable];
    result := True;
  except
    result := False;
  end;    
end;

procedure TTableProcessor.CreateATable(ATable : string);
var
  ANewTable : Variant;
  AField : Variant;
  AnIndex : Variant;
  AFieldDef : TAField;
  i, iPos : integer;
  sAuxStr, sIndexName : string;
begin
  ANewTable := FDatabase.CreateTableDef(ATable);
  if assigned(FOnLogMessage) then
    FOnLogMessage('Created Table ' + ATable);
  for i := 0 to FFieldList.Count - 1 do
  begin
    try
      AFieldDef := GetFieldProperties(FFieldList.Strings[i]);
      AField := ANewTable.CreateField(AFieldDef.FName, AFieldDef.FType, AFieldDef.FSize);
      AField.Required := AFieldDef.FRequired;
      try
        AField.AllowZeroLength := AFieldDef.FAllowZeroLength;
      except
      end;
      AField.Attributes := AFieldDef.FAttributes;
      AField.DefaultValue := AFieldDef.FDefaultValue;
      ANewTable.Fields.Append(AField);
      if assigned(FOnLogMessage) then
        FOnLogMessage('Table ' + ATable + ' - Added Field ' + AFieldDef.FName);
    except
      on e:exception do
      begin
        if assigned(FOnLogMessage) then
        begin
          FOnLogMessage('Table ' +ATable + ' Failed to create field + ' + AFieldDef.FName);
          FOnLogMessage(e.Message);
        end;
        raise;
      end;
    end;    
  end;
  FDatabase.TableDefs.Append(ANewTable);
  sIndexName := '';
  for i := 0 to FIndexList.count - 1 do
  begin
    sAuxStr := FIndexList.Strings[i];
    if sAuxStr = '' then
      continue;
    if sAuxStr = '-----' then
      break;
    iPos := Pos('I:', sAuxStr);
    if iPos > 0 then
    begin
      if sIndexName <> '' then
      begin
        ANewTable.Indexes.Append(AnIndex);
        sIndexName := '';
      end;
      sAuxStr := copy(sAuxStr, iPos + 2, length(sAuxStr));
      sAuxStr := GetAParameter(sAuxStr, sIndexName);
      AnIndex := ANewTable.CreateIndex(sIndexName);
      if Pos('Primary Key', FIndexList.Strings[i]) > 0 then
        AnIndex.Primary := -1;
      if Pos('Unique', FIndexList.Strings[i]) > 0 then
        AnIndex.Unique := -1;
      if Pos('Required', FIndexList.Strings[i]) > 0 then
        AnIndex.Required := -1;
      continue;
    end;
    if sIndexName <> '' then
      AnIndex.Fields.Append(ANewTable.CreateField(sAuxStr));
  end;
  if sIndexName <> '' then
  begin
    ANewTable.Indexes.Append(AnIndex);
    if assigned(FOnLogMessage) then
    begin
      FOnLogMessage('Table ' +ATable + ' - Created Index + ' + sIndexName);
    end;
  end;

end;

procedure TTableProcessor.GetFieldsAndIndexes(ATable : string);
var
  i : integer;
  iInitPos, iInitIndexes : integer;
  sAuxStr : string;
begin
  FFieldList.Clear;
  FIndexList.Clear;
  iInitPos := -1;

  //Get beginning of Table definitions
  for i := 0 to FScript.Count - 1 do
  begin
    if Pos('Table:', FScript.Strings[i]) >0 then
    begin
      sAuxStr := Trim(copy(FScript.Strings[i], 7, 100));
      if CompareText(sAuxStr, ATable) = 0 then
      begin
        iInitPos := i;
        break;
      end;
    end;
  end;
  if iInitPos < 0 then
    exit;
  iInitIndexes := 0;

  //Getting Fields
  for i := iInitPos + 1 to FScript.Count - 1 do
  begin
    sAuxStr := Trim(FScript.Strings[i]);
    if sAuxStr = '-----' then
      break;
    if (Pos('Fields', sAuxStr) > 0) and (Pos('Type', sAuxStr) > 0) and
      (Pos('Size', sAuxStr) > 0) then
    begin
      continue;
    end;
    if Pos('Indexes:', sAuxStr) > 0 then
    begin
      iInitIndexes := i;
      break;
    end;
    FFieldList.Add(sAuxStr);
  end;
  if iInitIndexes = 0 then
    exit;
  for i := iInitPos to FScript.Count - 1 do
  begin
    sAuxStr := Trim(FScript.Strings[i]);
    if sAuxStr = '-----' then
      break;
    if CompareText(sAuxStr, 'Indexes:') = 0 then
      continue;
    FIndexList.Add(sAuxStr);
  end;
  GetIndexes;

end;

function TTableProcessor.GetFieldProperties(AFieldLine : string) : TAField;
var
  sAuxStr, sField  : string;
  iPos: integer;
  AFieldDef : TAField;
begin
  AFieldDef.FType := 0;
  AFieldDef.FSize := 0;
  AFieldDef.FRequired := 0;
  AFieldDef.FAllowZeroLength := 0;
  AFieldDef.FAttributes := 0;
  AFieldDef.FDefaultValue := '';
  result := AFieldDef;
  sAuxStr := Trim(AFieldLine);
  sAuxStr := GetAParameter(sAuxStr, AFieldDef.FName);
  if sAuxStr = '' then
  begin
    result := AFieldDef;
    exit;
  end;
  sAuxStr := GetAParameter(sAuxStr, sField);
  AFieldDef.FType := StrToIntDef(sField, 0);
  if sAuxStr = '' then
  begin
    result := AFieldDef;
    exit;
  end;
  sAuxStr := GetAParameter(sAuxStr, sField);
  AFieldDef.FSize := StrToIntDef(sField, 0);
  if sAuxStr = '' then
  begin
    result := AFieldDef;
    exit;
  end;
  sAuxStr := GetAParameter(sAuxStr, sField);
  AFieldDef.FRequired := StrToIntDef(sField, 0);
  if sAuxStr = '' then
  begin
    result := AFieldDef;
    exit;
  end;
  sAuxStr := GetAParameter(sAuxStr, sField);
  AFieldDef.FAllowZeroLength := StrToIntDef(sField, 0);
  if sAuxStr = '' then
  begin
    result := AFieldDef;
    exit;
  end;
  sAuxStr := GetAParameter(sAuxStr, sField);
  AFieldDef.FAttributes := StrToIntDef(sField, 0);
  if sAuxStr = '' then
  begin
    result := AFieldDef;
    exit;
  end;
  sAuxStr := GetAParameter(sAuxStr, sField);
  AFieldDef.FDefaultValue := sField;
  result := AFieldDef;
end;

function TTableProcessor.GetAParameter(ALine : string; var AParam : string) : string;
var
  iPos : integer;
begin
  result := '';
  ALine := Trim(ALine);
  iPos := Pos(' ', Aline);
  if iPos = 0 then
  begin
    result := '';
    AParam := ALine;
  end else
  begin
    AParam := Copy(ALine, 1, iPos - 1);
    result := Trim(copy(ALine, iPos + 1, Length(Aline)));
  end;
end;

procedure TTableProcessor.EmptyIndexObjects;
var
  AnObject : TAnIndex;
  i : integer;
begin
  for i := 0 to FIndexObjectsList.Count - 1 do
  begin
    try
      AnObject := TAnIndex(FIndexObjectsList.Objects[i]);
      AnObject.Free;
    except
    end;
  end;
  FIndexObjectsList.Clear;
end;

procedure TTableProcessor.ModifyTable(ATable : string);
begin
  AddNewFields(ATable);
  AddNewIndexes(ATable);
  {if FbModifyFields then
    ModifyExistingFields(ATable);   }
end;

procedure TTableProcessor.AddNewIndexes(ATable : string);
var
  AnObject : TAnIndex;
  AnIndex : Variant;
  sIndex : string;
  i, j : integer;
  AFieldList : TStringList;
begin
  AFieldList := TStringList.Create;
  FTable := FDatabase.TableDefs[ATable];
  for i := 0 to FIndexObjectsList.Count - 1 do
  begin
    try
      AnObject := TAnIndex(FIndexObjectsList.Objects[i]);
      sIndex := FIndexObjectsList.strings[i];
      AnIndex := FTable.Indexes[sIndex];
      continue;
    except
    end;
    try
      AFieldList.Clear;
      AnIndex := FTable.CreateIndex(sIndex);
      AnIndex.Primary := AnObject.FPrimaryKey;
      AnIndex.Unique := AnObject.FUnique;
      AnIndex.Required := AnObject.FRequired;
      AFieldList.CommaText := AnObject.FFieldList;
      for j := 0 to AFieldList.Count - 1 do
      begin
        if AFieldList.Strings[j] <> '' then
          AnIndex.Fields.Append(FTable.CreateField(AFieldList.Strings[j]));
      end;
      FTable.Indexes.Append(AnIndex);
      if assigned(FOnLogMessage) then
      begin
        FOnLogMessage('Table ' +ATable + ' - Created Index + ' + sIndex);
      end;
    except
      on e:exception do
      begin
        if assigned(FOnLogMessage) then
        begin
          FOnLogMessage('Table ' +ATable + ' - Failed to create index + ' + sIndex);
          FOnLogMessage(e.Message);
        end;
      end;
    end;  
  end;

end;


//An experiment that did not work...


procedure TTableProcessor.ModifyExistingFields(ATable : string);
var
  AField, AFieldNew : Variant;
  AFieldDef : TAField;
  i, iPos, iOrdinalPosition : integer;
  sAuxStr, sIndexName : string;
begin
{  FTable := FDatabase.TableDefs[ATable];
  for i := 0 to FFieldList.Count - 1 do
  begin
    try
      AFieldDef := GetFieldProperties(FFieldList.Strings[i]);
      AField := FTable.Fields[AFieldDef.FName];

      //AField.Name := AField.Name + 'OLD'
      AFieldNew := FTable.CreateField(AFieldDef.FName + 'OLD', AFieldDef.FType, AFieldDef.FSize);
      AFieldNew.Required := AFieldDef.FRequired;
      try
        AFieldNew.AllowZeroLength := AFieldDef.FAllowZeroLength;
      except
      end;
      AFieldNew.Attributes := AFieldDef.FAttributes;
      AFieldNew.DefaultValue := AFieldDef.FDefaultValue;
      AFieldNew.OrdinalPosition := AField.OrdinalPosition;
      FTable.Fields.Append(AFieldNew);
    except
      Continue;
    end;
  end;   }
end;

procedure TTableProcessor.AddNewFields(ATable : string);
var
  AField : Variant;
  AFieldDef : TAField;
  i, iPos : integer;
  sAuxStr, sIndexName : string;
begin
  FTable := FDatabase.TableDefs[ATable];
  for i := 0 to FFieldList.Count - 1 do
  begin
    try
      AFieldDef := GetFieldProperties(FFieldList.Strings[i]);
      AField := FTable.Fields[AFieldDef.FName];
      continue;
    except
    end;
    try
      AField := FTable.CreateField(AFieldDef.FName, AFieldDef.FType, AFieldDef.FSize);
      AField.Required := AFieldDef.FRequired;
      try
        AField.AllowZeroLength := AFieldDef.FAllowZeroLength;
      except
      end;
      AField.Attributes := AFieldDef.FAttributes;
      AField.DefaultValue := AFieldDef.FDefaultValue;
      FTable.Fields.Append(AField);
      if assigned(FOnLogMessage) then
        FOnLogMessage('Table ' + ATable + ' - Added Field ' + AFieldDef.FName);
    except
      on e:exception do
      begin
        if assigned(FOnLogMessage) then
        begin
          FOnLogMessage('Table ' +ATable + ' Failed to create field + ' + AFieldDef.FName);
          FOnLogMessage(e.Message);
        end;
        raise;
      end;
    end;  
  end;
end;

procedure TTableProcessor.GetIndexes;
var
  i, iPos : integer;
  sAuxStr, sIndexName : string;
  AnObject : TAnIndex;
begin
  EmptyIndexObjects;
  sIndexName := '';
  for i := 0 to FIndexList.count - 1 do
  begin
    sAuxStr := FIndexList.Strings[i];
    if sAuxStr = '' then
      continue;
    if sAuxStr = '-----' then
      break;
    iPos := Pos('I:', sAuxStr);
    if iPos > 0 then
    begin
      sAuxStr := copy(sAuxStr, iPos + 2, length(sAuxStr));
      sAuxStr := GetAParameter(sAuxStr, sIndexName);
      AnObject := TAnIndex.create;
      AnObject.FName := sIndexName;
      AnObject.FUnique := False;
      AnObject.FRequired := False;
      AnObject.FPrimaryKey := False;
      AnObject.FFieldList := '';
      FIndexObjectsList.Add(sIndexName);
      FIndexObjectsList.Objects[FIndexObjectsList.Count - 1] := TObject(AnObject);
      if Pos('Primary Key', FIndexList.Strings[i]) > 0 then
        AnObject.FPrimaryKey := True;
      if Pos('Unique', FIndexList.Strings[i]) > 0 then
        AnObject.FUnique :=  True;
      if Pos('Required', FIndexList.Strings[i]) > 0 then
        AnObject.FRequired := True;
      continue;
    end;
    if sIndexName <> '' then
      AnObject.FFieldList := AnObject.FFieldList + ',' + sAuxStr;
  end;


end;


end.
