{
----------------------------------------------------------
MAS-CompMaker was used to generate this code
MAS-CompMaker, copyright(c) 2000, Mats Asplund / MAs Prod.
----------------------------------------------------------

Component Name: TmShortcut
        Author: Mats Asplund
      Creation: 2001-06-17
       Version: 1.0
   Description: A shortcut-producer component.
        Credit:
        E-mail: masprod@telia.com
          Site: http://go.to/masdp
  Legal issues: Copyright (C) 2001 by Mats Asplund


         Usage: This software is provided 'as-is', without any express or
                implied warranty.  In no event will the author be held liable
                for any  damages arising from the use of this software.

                Permission is granted to anyone to use this software for any
                purpose, including commercial applications, and to alter it
                and redistribute it freely, subject to the following
                restrictions:

                1. The origin of this software must not be misrepresented,
                   you must not claim that you wrote the original software.
                   If you use this software in a product, an acknowledgment
                   in the product documentation would be appreciated but is
                   not required.

                2. Altered source versions must be plainly marked as such, and
                   must not be misrepresented as being the original software.

                3. This notice may not be removed or altered from any source
                   distribution.

                4. If you decide to use this software in any of your applications.
                   Send me an EMail and tell me about it.

Quick Reference:
                TmShortcut inherits from TComponent.

                Key-Properties:
                  FullExePath:     The path + filename the shortcut should
                                   point at.

                  ShortcutFolder:  The folder in which the shortcut should
                                   be put.

                  ExeParameters:   If you wan't to send inparameters, then put
                                   them in here with <space> inbetween.

                  ShortcutCaption: The shortcut-name.

                  Destination:     Where the shortcut should be put. On desktop,
                                   startmenu, in spec. folder or all of them.

                Key-Methods:
                  Execute:         Creates the shortcut.

--------------------------------------------------------------------------------
}
unit mShortcut;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls, ShlObj, ActiveX, ComObj, Registry;

type
  TShortcutDest = (sdDeskTop, sdStartMenu, sdFolder, sdAll);

  TmShortcut = class(TComponent)
  private
    fShortcutDest: TShortcutDest;
    fFullExePath: TFileName;
    fShortcutFolder: string;
    fExeParameters: string;
    fShortcutCaption: string;
    fCopyright: string;
    procedure CreateShortcut(FullExePath, ShortcutFolder: TFileName;
      ExeParameters, ShortcutCaption: string);
    procedure SetCop(Value: string);
    procedure SetShortcutDest(const Value: TShortcutDest);
  { Private declarations }
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Execute;
  { Private declarations }
  published
    property FullExePath: TFileName read fFullExePath write fFullExePath;
    property ShortcutFolder: string read fShortcutFolder write fShortcutFolder;
    property ExeParameters: string read fExeParameters write fExeParameters;
    property ShortcutCaption: string read fShortcutCaption write fShortcutCaption;
    property Destination: TShortcutDest read fShortcutDest write SetShortcutDest;
    property Copyright: string read fCopyright write SetCop;
  { Published declarations }
  end;


procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('MAs Prod.', [TmShortcut]);
end;

constructor TmShortcut.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  fShortcutDest:= sdDeskTop;
  fCopyright:= 'Copyright (c) 2001, MAs Prod. / Mats Asplund';
end;

destructor TmShortcut.Destroy;
begin
  inherited Destroy;
end;

procedure TmShortcut.SetCop(Value: string);
begin
  Exit;
end;

procedure TmShortcut.CreateShortcut(FullExePath, ShortcutFolder: TFileName; ExeParameters, ShortcutCaption: string);
var
  MyObject: IUnknown;
  MySLink: IShellLink;
  MyPFile: IPersistFile;
  Directory: string;
  WFileName: WideString;
  MyReg: TRegIniFile;
begin
  try
    MyObject:= CreateComObject(CLSID_ShellLink);
    MySLink:= MyObject as IShellLink;
    MyPFile:= MyObject as IPersistFile;
    with MySLink do begin
//    Full path to exe-file shortcut is pointing at
      SetPath(PChar(FullExePath));
//    The parameters to be sent
      SetArguments(PChar(ExeParameters));
//    Set icon to exefile
      SetIconLocation(PChar(FullExePath), 0);
//    Set working dir
      SetWorkingDirectory(PChar(ExtractFilePath(FullExePath)));
    end;
    MyReg:= TRegIniFile.Create(
      'Software\MicroSoft\Windows\CurrentVersion\Explorer');

//  Put the shortcut on desktop
    if (fShortcutDest = sdDeskTop) or (fShortcutDest = sdAll) then
    begin
      Directory:= MyReg.ReadString('Shell Folders', 'Desktop', '');
      WFileName:= Directory + '\' + fShortcutCaption + '.lnk';
      MyPFile.Save(PWChar(WFileName), false);
    end;

//  Put the shortcut on start menu
    if (fShortcutDest = sdStartMenu) or (fShortcutDest = sdAll) then
    begin
      Directory:= MyReg.ReadString('Shell Folders', 'Start Menu', '');
      CreateDir(Directory);
      WFileName:= Directory + '\' + fShortcutCaption + '.lnk';
      MyPFile.Save(PWChar(WFileName), false);
    end;

//  Put the shortcut in spec. folder
    if (fShortcutDest = sdFolder) or (fShortcutDest = sdAll) then
    begin
      WFileName:= fShortcutFolder + '\' + fShortcutCaption + '.lnk';
      try
        MyPFile.Save(PWChar(WFileName), false);
      except
        on E: Exception do
          Showmessage(E.Message);
      end;
    end;
  finally
    MyReg.Free;
  end;
end;

procedure TmShortcut.SetShortcutDest(const Value: TShortcutDest);
begin
  if ((Value = sdFolder) or (Value = sdAll)) and (fShortcutFolder = '') then
  begin
    MessageDlg('You must specify a folder for the shortcut !', mtWarning, [mbOK], 0);
    fShortcutDest:= sdDeskTop;
    Exit;
  end
  else
    fShortcutDest:= Value;
end;

procedure TmShortcut.Execute;
begin
  CreateShortcut(fFullExePath, fShortcutFolder, fExeParameters, fShortcutCaption);
end;

end.

