{
----------------------------------------------------------
MAS-CompMaker was used to generate this code
MAS-CompMaker, copyright(c) 2001, Mats Asplund / MAs Prod.
----------------------------------------------------------

Component Name: TmWImage
        Author: Mats Asplund
      Creation: 2001-03-30
       Version: 1.0
   Description: An animated image-component to be shown when waiting.
        Credit:
        E-mail: masprod@telia.com
          Site: http://w1.545.telia.com/~u54503556 or http://go.to/masdp
  Legal issues: Copyright (C) 2001 by Mats Asplund


         Usage: This software is provided 'as-is', without any express or
                implied warranty.  In no event will the author be held liable
                for any  damages arising from the use of this software.

                Permission is granted to anyone to use this software for any
                purpose, including commercial applications, and to alter it
                and redistribute it freely, subject to the following
                restrictions:

                1. The origin of this software must not be misrepresented,
                   you must not claim that you wrote the original software.
                   If you use this software in a product, an acknowledgment
                   in the product documentation would be appreciated but is
                   not required.

                2. Altered source versions must be plainly marked as such, and
                   must not be misrepresented as being the original software.

                3. This notice may not be removed or altered from any source
                   distribution.

                4. If you decide to use this software in any of your applications.
                   Send me an EMail and tell me about it.

Quick Reference:
    TmWImage inherits from TImage.

    Key-Properties:
      property Interval: Integer read fInterval write fInterval;
      property ImageType: TImageType read fImageType write SetImageType;
      TImageType = (itWheel, itClock, itCompress, itSleep, itTypeWriter, itPen);

    Key-Methods:
      procedure Start;
      procedure Stop;

--------------------------------------------------------------------------------
}
unit mWImage;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls;

type
  TImageType = (itWheel, itClock, itCompress, itSleep, itTypeWriter, itPen);

  TmWImage = class(TImage)
  private
    FCopyright: string;
    fInterval: Integer;
    fImageType: TImageType;
    fImageNo: Integer;
    fTimer: TTimer;
    procedure ChangeImage(Sender: TObject);
    procedure SetCop(Value: string);
    procedure SetImageType(const Value: TImageType);
  { Private declarations }
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Start;
    procedure Stop;
  { Private declarations }
  published
    property Interval: Integer read fInterval write fInterval;
    property ImageType: TImageType read fImageType write SetImageType;
    property Copyright: string read FCopyright write SetCop;
  { Published declarations }
  end;


procedure Register;

implementation
{$R *.DCR}

const
  ImageRes1: array[1..2] of PChar = ('WHEEL1', 'WHEEL2');
  ImageRes2: array[1..3] of PChar = ('CLOCK1', 'CLOCK2', 'CLOCK3');
  ImageRes3: array[1..3] of PChar = ('ZIP1', 'ZIP2', 'ZIP3');
  ImageRes4: array[1..2] of PChar = ('SLEEP1', 'SLEEP2');
  ImageRes5: array[1..3] of PChar = ('TWRITER1', 'TWRITER2', 'TWRITER3');
  ImageRes6: array[1..3] of PChar = ('PEN1', 'PEN2', 'PEN3');

constructor TmWImage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  AutoSize:= true;
  Transparent:= true;
  fImageType:= itWheel;
  Picture.Bitmap.Handle:= LoadBitmap(HInstance, ImageRes1[1]);
  fTimer:= TTimer.Create(Self);
  fTimer.Enabled:= false;
  fInterval:= 1000;
  fTimer.Interval:= fInterval;
  FCopyright:= 'Copyright (c) 2001, MAs Prod. / Mats Asplund';
end;

destructor TmWImage.Destroy;
begin
  inherited Destroy;
end;

procedure TmWImage.SetCop(Value: string);
begin
  Exit;
end;

procedure TmWImage.Start;
begin
  fImageNo:= 1;
  fTimer.Interval:= fInterval;
  fTimer.OnTimer:= ChangeImage;
  fTimer.Enabled:= true;
end;

procedure TmWImage.Stop;
begin
  fTimer.Enabled:= false;
end;

procedure TmWImage.ChangeImage(Sender: TObject);
begin
  case fImageType of
    itWheel:
      case fImageNo of
        1:
          begin
            Picture.Bitmap.Handle:= LoadBitmap(HInstance, ImageRes1[1]);
            fImageNo:= 2;
          end;
        2:
          begin
            Picture.Bitmap.Handle:= LoadBitmap(HInstance, ImageRes1[2]);
            fImageNo:= 1;
          end;
      end;
    itClock:
      case fImageNo of
        1:
          begin
            Picture.Bitmap.Handle:= LoadBitmap(HInstance, ImageRes2[1]);
            fImageNo:= 2;
          end;
        2, 4:
          begin
            Picture.Bitmap.Handle:= LoadBitmap(HInstance, ImageRes2[2]);
            if fImageNo = 4 then fImageNo:= 1 else fImageNo:= 3;
          end;
        3:
          begin
            Picture.Bitmap.Handle:= LoadBitmap(HInstance, ImageRes2[3]);
            fImageNo:= 4;
          end;
      end;
    itCompress:
      case fImageNo of
        1:
          begin
            Picture.Bitmap.Handle:= LoadBitmap(HInstance, ImageRes3[1]);
            fImageNo:= 2;
          end;
        2:
          begin
            Picture.Bitmap.Handle:= LoadBitmap(HInstance, ImageRes3[2]);
            fImageNo:= 3;
          end;
        3:
          begin
            Picture.Bitmap.Handle:= LoadBitmap(HInstance, ImageRes3[3]);
            fImageNo:= 1;
          end;
      end;
    itSleep:
      case fImageNo of
        1:
          begin
            Picture.Bitmap.Handle:= LoadBitmap(HInstance, ImageRes4[1]);
            fImageNo:= 2;
          end;
        2:
          begin
            Picture.Bitmap.Handle:= LoadBitmap(HInstance, ImageRes4[2]);
            fImageNo:= 1;
          end;
      end;
    itTypeWriter:
      case fImageNo of
        1:
          begin
            Picture.Bitmap.Handle:= LoadBitmap(HInstance, ImageRes5[1]);
            fImageNo:= 2;
          end;
        2:
          begin
            Picture.Bitmap.Handle:= LoadBitmap(HInstance, ImageRes5[2]);
            fImageNo:= 3;
          end;
        3:
          begin
            Picture.Bitmap.Handle:= LoadBitmap(HInstance, ImageRes5[3]);
            fImageNo:= 1;
          end;
      end;
    itPen:
      case fImageNo of
        1:
          begin
            Picture.Bitmap.Handle:= LoadBitmap(HInstance, ImageRes6[1]);
            fImageNo:= 2;
          end;
        2:
          begin
            Picture.Bitmap.Handle:= LoadBitmap(HInstance, ImageRes6[2]);
            fImageNo:= 3;
          end;
        3:
          begin
            Picture.Bitmap.Handle:= LoadBitmap(HInstance, ImageRes6[3]);
            fImageNo:= 1;
          end;
      end;
  end;
  Invalidate;
end;

procedure TmWImage.SetImageType(const Value: TImageType);
begin
  fImageType:= Value;
  case fImageType of
    itWheel:
      Picture.Bitmap.Handle:= LoadBitmap(HInstance, ImageRes1[1]);
    itClock:
      Picture.Bitmap.Handle:= LoadBitmap(HInstance, ImageRes2[2]);
    itCompress:
      Picture.Bitmap.Handle:= LoadBitmap(HInstance, ImageRes3[1]);
    itSleep:
      Picture.Bitmap.Handle:= LoadBitmap(HInstance, ImageRes4[1]);
    itTypeWriter:
      Picture.Bitmap.Handle:= LoadBitmap(HInstance, ImageRes5[1]);
    itPen:
      Picture.Bitmap.Handle:= LoadBitmap(HInstance, ImageRes6[1]);
  end;
end;

procedure Register;
begin
  RegisterComponents('MAs Prod.', [TmWImage]);
end;

end.

